package es.uv.garcosda.controllers;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.web.PagedResourcesAssembler;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.PagedModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import es.uv.garcosda.domain.Video;
import es.uv.garcosda.services.VideoService;


@RestController
@RequestMapping("/api/v1/videos")
public class VideoController {
	private final static Logger LOGGER = LoggerFactory.getLogger(VideoController.class);
	
	@Autowired private VideoService vs;
	@Autowired private PagedResourcesAssembler<Video> pagedResourcesAssembler;
	
	@GetMapping("")
	public ResponseEntity<?> findVideos(@RequestParam(value="title", defaultValue = "", required = false) String title, 
			@RequestParam(value="user", defaultValue = "", required = false) String userId,
			@RequestParam(value="page", defaultValue = "0", required = false) int page,
		    @RequestParam(value="size", defaultValue = "0", required = false) int size) {
		
		List<Video> videos = new ArrayList<Video>();
		Page<Video> page_videos = Page.empty();
		Pageable pageable;
		if(size == 0) {
			pageable = PageRequest.of(0, 1);
		}
		else {
			pageable = PageRequest.of(page, size);
		}
		
		if(title.isBlank() && userId.isBlank()) {
			LOGGER.debug("Get all videos");
			if(size == 0) {
				videos = vs.findAll();
			}
			else {
				page_videos = vs.findAll(pageable);
			}	
		}
		else if(!title.isBlank()) {
			LOGGER.debug("Get videos by title");
			videos = vs.findByTitle(title);
		}
		else if(!userId.isBlank()) {
			LOGGER.debug("Get videos by user");
			videos = vs.findByUserId(userId);
		}
		
		if(size == 0) {
			for (Video v : videos) {
				v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 0)).withRel("collection"));
				v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 10)).withRel("page_videos"));
				v.add(linkTo(methodOn(VideoController.class).findVideoById(v.getId().toString())).withSelfRel());
				v.add(linkTo(methodOn(UserController.class).findUserById(v.getUserId().toString())).withRel("user"));
				v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 0)).withRel("all_comments"));
				v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 10)).withRel("page_comments"));
			}
			return new ResponseEntity<List<Video>>(videos, HttpStatus.OK);
		}
		else {
			for (Video v : page_videos.getContent()) {
				v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 0)).withRel("collection"));
				v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 10)).withRel("page_videos"));
				v.add(linkTo(methodOn(VideoController.class).findVideoById(v.getId().toString())).withSelfRel());
				v.add(linkTo(methodOn(UserController.class).findUserById(v.getUserId().toString())).withRel("user"));
				v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 0)).withRel("all_comments"));
				v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 10)).withRel("page_comments"));
			}
			return new ResponseEntity<PagedModel<EntityModel<Video>>>(pagedResourcesAssembler.toModel(page_videos), HttpStatus.OK);
		}
		
		
	}
	
	@GetMapping("{id}")
	public ResponseEntity<Video> findVideoById(@PathVariable("id") String id) {
		LOGGER.debug("Get video id: "+id);
		Optional<Video> video = vs.findById(id);
		
		if(!video.isEmpty()) {
			Video v = video.get();
			v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 0)).withRel("collection"));
			v.add(linkTo(methodOn(VideoController.class).findVideos("", "", 0, 10)).withRel("page_videos"));
			v.add(linkTo(methodOn(VideoController.class).findVideoById(id)).withSelfRel());
			v.add(linkTo(methodOn(UserController.class).findUserById(v.getUserId().toString())).withRel("user"));
			v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 0)).withRel("all_comments"));
			v.add(linkTo(methodOn(CommentController.class).findComments(v.getId(), "", 0, 10)).withRel("page_comments"));
			return new ResponseEntity<Video>(video.get(), HttpStatus.OK);
		}
		else {
			return new ResponseEntity<Video>(new Video(), HttpStatus.NOT_FOUND);
		}
	}
	
	@PostMapping("")
	public ResponseEntity<Video> createVideo(@RequestBody Video video) {
		LOGGER.debug("Create video");
		Video createdVideo = vs.create(video);
		
		if(createdVideo.getId() != null) {
			createdVideo.add(linkTo(methodOn(VideoController.class).findVideoById(createdVideo.getId().toString())).withSelfRel());
			createdVideo.add(linkTo(methodOn(UserController.class).findUserById(createdVideo.getUserId().toString())).withRel("user"));
			return new ResponseEntity<Video>(createdVideo, HttpStatus.OK);
		}
		else {
			return new ResponseEntity<Video>(new Video(), HttpStatus.BAD_REQUEST);
		}
	}
			
	@DeleteMapping("{id}")
	public ResponseEntity<String> deleteVideoById(@PathVariable("id") String id) {
		LOGGER.debug("Delete video id: "+id);
		vs.deleteById(id);
		return new ResponseEntity<String>("", HttpStatus.OK);
	}
	
}
